using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;
using System.Text.RegularExpressions;
using System.IO.Ports;
using System.Threading;



namespace COMPortTester
{
    delegate void OutPutLineDelegate();

    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            // Populate the COM port list
            List<ComData>   comData = GetUartList();
            for (int i = 0; i< comData.Count ; ++i)
            {
                COMPortsList.Items.Add( comData[i].ComName + " " + comData[i].ComType);                
            }
            COMPortsList.SelectedIndex = COMPortsList.Items.Count - 1;

            // Populate BaudRate
            // To get all the possible Baud rates please read the x300-Man.pdf
            BaudRateList.Items.Add(115200);
            BaudRateList.Items.Add(57600);
            BaudRateList.Items.Add(38400);
            BaudRateList.Items.Add(19200);
            BaudRateList.Items.Add(9600);
            BaudRateList.SelectedItem = 115200;
        }
        private struct ComData
        {
            public string ComName,ComType;
            public ComData(string Comname, string Comtype)
            {
                ComName = Comname;
                ComType = Comtype;
            }            
        };

        private List<ComData> GetUartList()
        {
            List<ComData>   RetVal = new List<ComData>();
            string[]    subKeyNames;
            RegistryKey rk = Registry.LocalMachine;
            RegistryKey sk = rk.OpenSubKey("Drivers\\Active");
            rk.Close();
            if (sk == null)
            {
                return null;
            }
            subKeyNames = sk.GetSubKeyNames();
            foreach (string keyName in subKeyNames)
            {
                string valueContents;
                RegistryKey subKey = sk.OpenSubKey(keyName);
                if (subKey == null)
                    continue;
                try
                {
                    valueContents = subKey.GetValue("Name").ToString();
                }
                catch(NullReferenceException)
                {
                    // No use of a device without a name
                    continue;
                }
                if (valueContents != null &&
                    String.Compare(valueContents.Substring(0,3),"COM") == 0)
                {
                    string FriendlyName = null;
                    //Check fot the Frienldy Name
                    try
                    {
                        string builtInName = subKey.GetValue("Key").ToString();
                        RegistryKey regk = Registry.LocalMachine;
                        RegistryKey builtinKey = regk.OpenSubKey(builtInName);
                        FriendlyName = builtinKey.GetValue("FriendlyName").ToString();
                        rk.Close();
                        builtinKey.Close();
                    }
                    catch (NullReferenceException)
                    {
                        // Must be a null string
                        FriendlyName = "Unknown";
                    }
                        
                    RetVal.Add(new ComData(valueContents, FriendlyName));
                }
                subKey.Close();
            }   
            sk.Close();
            return RetVal;
        }

        private static SerialPort serialPort = new SerialPort();
        static bool _continue;
        static string lineRead;
        private Thread readThread;

        private void OpenBtn_Click(object sender, EventArgs e)
        {
            object ComName = COMPortsList.SelectedItem;
            Int32 ComBaud = Convert.ToInt32(BaudRateList.SelectedItem);
            if (ComName == null)
            {
                MessageBox.Show("Please select a COM port from the list");
                return;
            }
            if (ComBaud == 0)
            {
                MessageBox.Show("Please choose a baud rate");
                return;
            }
            serialPort.BaudRate = ComBaud;
            serialPort.PortName = ComName.ToString().Substring(0, 4); //COM#
            serialPort.Parity = System.IO.Ports.Parity.None;
            serialPort.StopBits = System.IO.Ports.StopBits.One;
            serialPort.DataBits = 8;
            serialPort.ReadTimeout = 5000;
            serialPort.WriteTimeout = 500;
            try
            {
                serialPort.Open();
                COMPortsList.Enabled = false;
                BaudRateList.Enabled = false;
                _continue = true;
                readThread = new Thread(Read);
                OpenBtn.Enabled = false;
                readThread.Start();
                CloseBtn.Enabled = true;
            }
            catch (ThreadStateException exc)
            {
                MessageBox.Show(exc.ToString());
            }
            catch (OutOfMemoryException ex)
            {
                MessageBox.Show(ex.ToString());
            }
            catch (Exception) { }
        }

        private void CloseBtn_Click(object sender, EventArgs e)
        {
            try
            {
                _continue = false;
                readThread.Join();
                serialPort.Close();
                COMPortsList.Enabled = true;
                BaudRateList.Enabled = true;
                OpenBtn.Enabled = true;
                CloseBtn.Enabled = false;
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.ToString());
            }
        }

        private void TestBtn_Click(object sender, EventArgs e)
        {
            char    []buf = new char[10];
            _continue = true;
            

            if (serialPort.IsOpen == false)
                return;

            try
            {
                serialPort.Write("Test");
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.ToString());
            }
        }

        private delegate void UpdateLineDelegate(string msg);
        private void updateLine(string msg)
        {
            if (msg == null)
                OutputLines.Items.Add("Timeout");
            else
                OutputLines.Items.Add(lineRead);
        }

        public void Read()
        {
            UpdateLineDelegate d = new UpdateLineDelegate(this.updateLine);
            while (_continue) 
            {
                try
                {
                    if (serialPort.IsOpen && serialPort.BytesToRead > 0)
                    {
                        lineRead = serialPort.ReadExisting().ToString();
                        this.Invoke(d, lineRead);
                    }
                    Thread.Sleep(10);
                }
                catch (TimeoutException){}
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (serialPort.IsOpen)
                serialPort.Close();
        }

    }
}