using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace GpioDotNet
{
    /******************************************************************************************
    * This short program demonstrates how to use the .Net wrapper for Compulab's GPIO Driver.
    * Pin 12 on P13(GPIO122) & Pin 14 on P13(GPIO124) should be connected for correct behavior.
    * GPIO122 can be changed by GPIOX_128 in order to use GPIO EXTENDER functionality.
    * In addition, the voltage level may be monitored with a DVM or an oscilloscope.
    ******************************************************************************************/
    class Program
    {
        internal const uint GPIO_122  =   122;	//KP_MKOUT1 - Pin 12 on P13 connector located on the SB-X300
        internal const uint GPIO_124  =   124;	//KP_MKOUT3 - Pin 14 on P13 connector located on the SB-X300
        internal const uint GPIOX_128 =   128;  //From the GPIO EXTENDER - Pin 24 on P11 connector located on the SB-X300

        public static IntPtr gpioEventHandle = IntPtr.Zero;
        public static GpioCmX300 hGPIO;
        public static Thread t;

        public static void GpioEventThread()
        {
            while (true)
            {
                NativeMethods.WaitForSingleObject(gpioEventHandle, NativeMethods.INFINITE);
                System.Console.WriteLine("Interrupt Triggered!");
                Thread.Sleep(1);
                //acknowledges recieved interrupt
                hGPIO.AckInterrupt(GPIO_124);
                System.Console.WriteLine("Interrupt Acknowledged!");
            }
        }

        static void Main(string[] args)
        {
            //Creates an instance of the gpio driver .Net wrapper
            hGPIO = new GpioCmX300();


            hGPIO.SetAsOutput(128, GpioLevel.LOW);
            
            
            return;
            
            //Sets gpio 124 as Input
            hGPIO.SetAsInput(GPIO_124);

            //sets gpio 122 as Output with a logical value of 0
            hGPIO.SetAsOutput(GPIO_122, GpioLevel.LOW);

            //checks wether GPIO is setup high or low.
            if ((hGPIO.GetGpioLevel(GPIO_122)) == GpioLevel.HIGH)
                System.Console.WriteLine("Gpio" + GPIO_124 + ": HIGH");
            else
                System.Console.WriteLine("Gpio" + GPIO_124 + ": LOW");

            Thread.Sleep(1000);

            //sets gpio 122 as Output with a logical value of 1
            hGPIO.SetAsOutput(GPIO_122, GpioLevel.HIGH);

            //checks wether GPIO is setup high or low.
            if ((hGPIO.GetGpioLevel(GPIO_122)) == GpioLevel.HIGH)
                System.Console.WriteLine("Gpio" + GPIO_124 + ": HIGH");
            else
                System.Console.WriteLine("Gpio" + GPIO_124 + ": LOW");
            
            Thread.Sleep(1000);

            //sets gpio 122 as Output with a logical value of 0
            hGPIO.SetAsOutput(GPIO_122, GpioLevel.LOW);

            Thread.Sleep(1000);

            //Sets up an interrupt event for GPIO 124 to react on rising edge only.
            hGPIO.SetupInterrupt(GPIO_124, false, true);

            //The event must be named GpioEventX (X is the GPIO number). In this example GpioEvent170
            gpioEventHandle = NativeMethods.CreateEvent(IntPtr.Zero, false, false, ("GpioEvent" + GPIO_124));
            if (gpioEventHandle.Equals(IntPtr.Zero))
            {
                System.Console.WriteLine("Can't Create Event!");
            }

            //starts a new thread that waits for GPIO interrupts
            ThreadStart tStart = new ThreadStart(GpioEventThread);
            t = new Thread(tStart);
            t.Start();

            //The following code should generate interrupts for GPIO 124
            for (int i = 0; i < 3; i++)
            {
                //sets gpio 122 as Output with a logical value of 1
                hGPIO.SetAsOutput(GPIO_122, GpioLevel.HIGH);

                Thread.Sleep(1000);

                //sets gpio 122 as Output with a logical value of 0
                hGPIO.SetAsOutput(GPIO_122, GpioLevel.LOW);

                Thread.Sleep(1000);
            }


            //stops started thread and closes event handle
            t.Abort();
            NativeMethods.CloseHandle(gpioEventHandle);

            //disables the initialized interrupt
            hGPIO.DisableInterrupt(GPIO_124);
            System.Console.WriteLine("Interrupt Disabled!"); 
            
            
            Thread.Sleep(1000);

        }
    }
}
