using System;
using System.Collections.Generic;
using System.Text;

namespace SpiDotNet
{
    class Program
    {
        internal const uint SPI_PORT_NUMBER   = 1;


        internal const uint SSP3_FREQ		  =	4;	//Specified in KHz, accroding to spec needs to be between DC and 5MHz
        internal const uint	SSP3_DATA_LEN_EX  =	16;	//Each transaction is 16 bits (writing 16 and reading 16 minimum)
        internal const uint	SSP3_DATA_LEN	  =	8;	//Each transaction is 8 bits (writing 8 and reading 8 minimum)
        internal const uint BUFFER_SIZE       = 5;

        private static SpiCmX300 hSPI;

        static void ThermometerTransaction16BitsVersion() 
        {
            
            //Initial configuration of the ssp port according to the DS1722+ (MAXIM's) spec
            hSPI.SPIUserSetConfiguration(SSP3_DATA_LEN_EX,
                                         SSP3_FREQ,
                                         Polarity.LOW_INACTIVE,
                                         Polarity.LOW_INACTIVE, // Polarity of CS
                                         true,  // Will TX line be tristated when CS is deasserted
                                         true,
                                         false,
                                         0);    // Will the SSP port drive the Chip Select or is it an external GPIO we use - Here we use the internal controller

            uint[]	ReadBuffer  = new uint[BUFFER_SIZE];
            uint[]	WriteBuffer = new uint[BUFFER_SIZE];

            //Send the address 0x02 to read Temprature MSB
            WriteBuffer[0] = 0x200;

            //In one transaction we send information at the first half, and expect data at last 8 bits of the transaction
            uint numOfTransactions = 1;
           
            //Send the transaction
            hSPI.SPIUserTransact(numOfTransactions, WriteBuffer,ref ReadBuffer);

            //Only 16 bits are non zero at most because data frame size was configured to be 16 bits to read
            //And the hardware padds zeros for the higher bits.

            System.Console.WriteLine("The temprature is: 0x" + ReadBuffer[0].ToString("X"));

            //Now we will write to the configuration register
            //Using the same configuration
            ReadBuffer[0] = 0;
            WriteBuffer[0] = 0x80F0; //The address is the upper 8 bits and the data is the lower 8 bits

            hSPI.SPIUserTransact(numOfTransactions, WriteBuffer, ref ReadBuffer);

            //Let's see if the configuration was written correctly
            WriteBuffer[0] = 0x000; //the address of the config register when we want to read it

            hSPI.SPIUserTransact(numOfTransactions, WriteBuffer, ref ReadBuffer);

            System.Console.WriteLine("The config reg now is 0x" + ReadBuffer[0].ToString("X") + ", default after reset is 0xE3.");
        }
        static void ThermometerTransaction8BitsVersion()
        {
            //Initial configuration of the ssp port according to the DS1722+ (MAXIM's) spec
            hSPI.SPIUserSetConfiguration(SSP3_DATA_LEN,
                                     SSP3_FREQ,
                                     Polarity.LOW_INACTIVE,
                                     Polarity.LOW_INACTIVE, // Polarity of CS
                                     true,  // Will TX line be tristated when CS is deasserted
                                     true,
                                     false,
                                     0);    // Will the SSP port drive the Chip Select or is it an external GPIO we use - Here we use the internal controller

            //Ok now the port is configured and we can start transmitting

            uint[] ReadBuffer = new uint[BUFFER_SIZE];
            uint[] WriteBuffer = new uint[BUFFER_SIZE];

            
            uint numOfTransactions = 2;
            
            //Send the address 0x02 to read Temprature MSB
            WriteBuffer[0] = 0x200;

            //We still send something while we read data - but the device ignores this data
            WriteBuffer[1] = 0x0;
            
            //Send the transaction
            hSPI.SPIUserTransact(numOfTransactions, WriteBuffer, ref ReadBuffer);

            //The answer is in index 1 because we sent data in cycle 0 and read the data the next cycle
            uint temprature = ReadBuffer[1];

            ReadBuffer[0] = 0;
            ReadBuffer[1] = 0;

            numOfTransactions = 2;

            //Send the address 0x01 to read Temprature LSB
            WriteBuffer[0] = 0x1;
            //Send the transaction
            hSPI.SPIUserTransact(numOfTransactions, WriteBuffer, ref ReadBuffer);
            //NOTICE:Only 8 bits are non zero at most because data frame size was configured to be 8 bits to read
            //And the hardware padds zeros for the higher bits.

            System.Console.WriteLine("The temprature is: MSB=0x" + temprature.ToString("X") + ", LSB=0x" + ReadBuffer[1].ToString("X")+ ".");

            //Now we will write to the configuration register
            //Using the same configuration
            ReadBuffer[0] = 0;
            ReadBuffer[1] = 0;

            WriteBuffer[0] = 0x80; //The address 
            WriteBuffer[1] = 0xF0; //The data to be written

            hSPI.SPIUserTransact(numOfTransactions, WriteBuffer, ref ReadBuffer);

            //Let's see if the configuration was written correctly
            WriteBuffer[0] = 0x0; //the address of the config register when we want to read it
            WriteBuffer[1] = 0x0;
            hSPI.SPIUserTransact(numOfTransactions, WriteBuffer, ref ReadBuffer);

            System.Console.WriteLine("The config reg now is 0x" + ReadBuffer[1].ToString("X") + ", default after reset is 0xE3.");
        }

        static void Main(string[] args)
        {
            hSPI = new SpiCmX300(SPI_PORT_NUMBER);
            ThermometerTransaction16BitsVersion();
            ThermometerTransaction8BitsVersion();
            return;
        }
    }
}