// GPIOSample.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "Windev.h"

#include "gpio.h"


#define GPIO_122		122	//KP_MKOUT1 - Pin 12 on P13 connector located on the SB-X300
#define GPIO_124		124	//KP_MKOUT3 - Pin 14 on P13 connector located on the SB-X300
#define GPIOX_128		128	//From the GPIO EXTENDER - Pin 24 on P11 connector located on the SB-X300

/************************************************************************************
This short program demonstrates how to use Compulab's GPIO Driver.
************************************************************************************/


int _tmain(int argc, _TCHAR* argv[])
{
	int iRet = TRUE;
	HANDLE hEvent; 
	GPIO_VALUE gpioVal;
	
	//Opens a handle to the gpio driver
	HANDLE hGpio = GPIOOpen();
	if (hGpio == NULL)
	{
		printf("OOOOOOOOOOooooooo!\n");	
		Sleep(10000);
		return FALSE;
	}
	//Check if the gpio we are about to use are available for us
	//if (!GPIOIsAvailable(hGpio, GPIO_122) || !GPIOIsAvailable(hGpio,GPIO_124))
	//{
	//	printf("The gpios selected are not available\n");
	//	return 0;
	//}
	////Letting the driver know we are going to use these GPIO's
	//if (!GPIOAquireGpio(hGpio, GPIO_122) || !GPIOAquireGpio(hGpio, GPIO_124))
	//{
	//	printf("The gpios selected are not available\n");
	//	return 0;
	//}
		
	//Sets gpio 124 as Input
	GPIOSetAsInput(hGpio, GPIO_124);
	
	//sets gpio 122 as Output with a logical value of 1
	GPIOSetAsOutput(hGpio, GPIO_122, GPIO_LOW);

	Sleep(1000);

	GPIOSetAsOutput(hGpio, GPIO_122, GPIO_HIGH);

	Sleep(1000);

	GPIOSetAsOutput(hGpio, GPIO_122, GPIO_LOW);

	Sleep(1000);

	GPIOSetAsOutput(hGpio, GPIO_122, GPIO_HIGH);
	
	Sleep(1000);

	printf("That concludes the GPIO sample.\n");
	//Close the handle to the gpio driver

	//Sleep(1000);
	//GPIOClose(hGpio);

	//return 0;

	//Sets up an interrupt event for GPIO 124 to react on falling and rising edge
		
	hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	printf("Event hnd = 0x%x.\n",hEvent);

	//Initialize the GPIO_INTERRUPT struct
	GPIO_INTERRUPT intr;
	intr.dwGpioNum = GPIO_124;			
	intr.Edge = GPIO_REACT_ON_BOTH;		//Decides if we react on falling edge, rising edge or both.
	intr.Event = hEvent;				//The event the will be signaled when the edge will happen.
	iRet = GPIORegisterInterrupt(hGpio, intr);

	if (!iRet) {
		printf("Please contact Compulab if you ever reach this point with this demo\r\n");
		CloseHandle(hEvent);
		return -1;
	}

	GPIO_VALUE gpioVal122 = GPIO_HIGH;
	//This loop will continue until you trigger the edge event	
	while(TRUE)
	{
		//Waits for the edge event we just initialized. 
		iRet = WaitForSingleObject(hEvent, 3000);

		if (iRet == WAIT_TIMEOUT)
		{
			//No edge was detected	
			printf("GPIOSample: Interrupt timeout. Trigger an edge for gpio 124 to continue\n");
			//Invert the value of GPIO 122
			GPIOSetAsOutput(hGpio, GPIO_122, gpioVal122);
			gpioVal122 = (gpioVal122 == GPIO_HIGH) ? GPIO_LOW : GPIO_HIGH;
		}
		else
		{
			//Edge was detected. we can proceed 
			printf("GPIOSample: Interrupt event was signaled.\n");
			//This function will signal the kernel that interrupt processing has been completed.
			GPIOInterruptDone(hGpio, GPIO_124);
			break;
		}
	}

	//Unregisters the interrupt on gpio 124
	GPIOUnRegisterInterrupt(hGpio, GPIO_124);

	CloseHandle(hEvent);

	/*-----------------------------------------------------------------*/

	//This section demonstrates the same functionality only with a GPIO from the GPIO Extender
	GPIOSetAsInput(hGpio, GPIOX_128);
	
	//sets gpio 122 as Output with a logical value of 1
	GPIOSetAsOutput(hGpio, GPIO_122, GPIO_LOW);
	

	//Sets up an interrupt event for GPIO 124 to react on falling and rising edge
		
	hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	printf("Event hnd = 0x%x.\n",hEvent);

	//Initialize the GPIO_INTERRUPT struct	
	intr.dwGpioNum = GPIOX_128;		
	intr.Edge = GPIO_REACT_ON_BOTH;		//Decides if we react on falling edge, rising edge or both.
	intr.Event = hEvent;				//The event the will be signaled when the edge will happen.
	iRet = GPIORegisterInterrupt(hGpio, intr);

	if (!iRet) {
		printf("Please contact Compulab if you ever reach this point with this demo\r\n");
		CloseHandle(hEvent);
		return -1;
	}

	//This loop will continue until you trigger the edge event	
	while(TRUE)
	{
		//Waits for the edge event we just initialized. 
		iRet = WaitForSingleObject(hEvent, 3000);

		if (iRet == WAIT_TIMEOUT)
		{
			//No edge was detected	
			printf("GPIOSample: Interrupt timeout. Trigger an edge for gpio 124 to continue\n");
			//Invert the value of GPIO 122
			GPIOSetAsOutput(hGpio, GPIO_122, gpioVal122);
			gpioVal122 = (gpioVal122 == GPIO_HIGH) ? GPIO_LOW : GPIO_HIGH;
		}
		else
		{
			//Edge was detected. we can proceed 
			printf("GPIOSample: Interrupt event was signaled.\n");
			//This function will signal the kernel that interrupt processing has been completed.
			GPIOInterruptDone(hGpio, GPIOX_128);
			break;
		}
	}

	//Unregisters the interrupt on gpio 128
	GPIOUnRegisterInterrupt(hGpio, GPIOX_128);

	CloseHandle(hEvent);

	/*-----------------------------------------------------------------*/

	//This section demonstrates polling.
	//The loop will run until the logical value of gpio 124 is 0
	gpioVal = GPIO_LOW;
	//Will use GPIO_124 value to set GPIO_122 value
	GPIOSetAsOutput(hGpio, GPIO_124, GPIO_HIGH);
	GPIOSetAsInput(hGpio, GPIO_122);
	printf("Polling GPIO 122, Assert a logical value 1 to continue.\n");
	while(gpioVal != GPIO_HIGH)
	{
		if(GPIOGetValue(hGpio, GPIO_122, &gpioVal) == FALSE)
		{
			printf("Error: Unable to read gpio 124 value\n");
			break;
		}
		Sleep(100);		
	}

	//Letting the driver know we don't need the GPIO's anymore
	GPIOReleaseGpio(hGpio, GPIO_122);
	GPIOReleaseGpio(hGpio, GPIO_124);
	
	printf("That concludes the GPIO sample.\n");
	//Close the handle to the gpio driver
	GPIOClose(hGpio);

	return 0;
}

