//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
//
// (C) Copyright 2006 Marvell International Ltd.
// All Rights Reserved
//
/**
**  FILENAME:       I2C_DRV.h
**  
**  PURPOSE: contains all I2C driver specific macros, typedefs, and 
**           prototypes.  Declares no storage.
**  
**  
******************************************************************************/
#ifndef __I2C_DRV_H
#define __I2C_DRV_H

//
// IOCTL info, needs to be visible for application
//
#define I2C_IOCTL_INDEX								0x0810


#define IOCTL_I2C_TRANSACT								CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        I2C_IOCTL_INDEX + 0,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)




/* Speed mode */
/* Fast speed mode will be required in order that I2C operation can be completed more faster. 
For example, IPM will set fast speed mode during volatage and frequence changing for low power mode.
But please notice carefully when using fase speed, it will require all I2C client device in I2C bus have 
fast speed mode support capbility */
typedef enum
{
    PXA_I2C_FAST_SPEED = 1,
    PXA_I2C_NORMAL_SPEED
} PXA_I2C_SPEED_T;


typedef enum
{
	I2C_OPCODE_READ		=	1,
	I2C_OPCODE_WRITE	=	2,
} PXA_I2C_OPERATION_CODE;


typedef struct
{
    /* input: */
	DWORD			mTransactions;
	PXA_I2C_SPEED_T	mClkSpeed;
	UCHAR			mDeviceAddr;  //7 bit
	PXA_I2C_OPERATION_CODE			*mOpCode;
    DWORD			*mBufferOffset;
    DWORD			*mTransLen;
    /* output: */
    DWORD			mErrorCode;
    UCHAR			*mBuffer;
} PXA_I2CTRANS_T;

#define I2CTRANS_MAX_STREAMED_TRANSACTIONS  8

//------------------------------------------------------------------------------
//
//  Functions: I2Cxxx
//------------------------------------------------------------------------------
//
//  Functions: I2Cxxx for accessing I2C from user space via IOCTLs
//------------------------------------------------------------------------------

__inline HANDLE I2CUserOpen(TCHAR *strName)
{
    HANDLE hDevice;
	hDevice = CreateFile (strName, 0, 0, NULL, OPEN_ALWAYS | CREATE_NEW, 0, NULL);
    return hDevice;
}

__inline void I2CUserClose(HANDLE hContext)
{
	if(hContext == NULL)
		return;
	CloseHandle(hContext);
}
__inline BOOL I2CUserTransact(HANDLE hDevice, PXA_I2CTRANS_T* pTrans)
{
	BOOL iRet;
	DWORD BytesWritten;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl (hDevice, IOCTL_I2C_TRANSACT, NULL, 0, pTrans, sizeof(PXA_I2CTRANS_T), &BytesWritten, NULL);
	return (iRet && BytesWritten == sizeof(PXA_I2CTRANS_T));
}

#endif 
