/******************************************************************************
// INTEL CONFIDENTIAL
// Copyright 2000 ~ 2006 Intel Corporation All Rights Reserved. 
// The source code contained or described herein and all documents related to 
// the source code (Material) are owned by Intel Corporation or its 
// suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material contains trade secrets and 
// proprietary and confidential information of Intel or its suppliers and 
// licensors. The Material is protected by worldwide copyright and trade secret 
// laws and treaty provisions. No part of the Material may be used, copied, 
// reproduced, modified, published, uploaded, posted, transmitted, distributed, 
// or disclosed in any way without Intels prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
******************************************************************************/
#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "codecAC.h"
#include "misc.h"

typedef struct _Options{
	int sampleRate;
	int channels;
	int quality;
	int subbands;
	int blocks;
	int allocation;
	int bitpool;
} Options;

typedef unsigned short		WORD;
typedef unsigned int		DWORD;

typedef struct _WAVEHEADER{
	DWORD	dwRIFF;
	DWORD	dwTotalSize;
	DWORD	dwWAVE;
	DWORD	dwFMT;
	DWORD	dwFmtSize;
	WORD	wFormatTag;
	WORD	nChannels;
	DWORD	nSamplesPerSec;
	DWORD	nAvgBytesPerSec;
	WORD	nBlockAlign;
	WORD	wBitsPerSample;
	DWORD	dwDATA;
	DWORD	dwDataSize;
}WAVEHEADER;

void Help();
void ParseCmdLine(int argc, char **argv, Options *pOptions, FILE **fpi, FILE** fpo);

int main(int argc, char **argv)
{
	FILE  *fpi, *fpo;
	Options options;
	MiscGeneralCallbackTable   *pCallBackTable = NULL;
	IppSBCEncoderState *pSbcEncoderState = NULL;
	IppSBCConfig encoderConfig;
	IppSound sound;
	IppBitstream bitstream;
	unsigned int frameLength, frameNum;
	WAVEHEADER wavFmt;
	int FrameBytes, join;

	frameNum = 0;

	memset(&options, 0, sizeof(Options));
	ParseCmdLine(argc, argv, &options, &fpi, &fpo);

	fread(&wavFmt, 1, sizeof(WAVEHEADER), fpi);

	if(options.channels <0 || options.channels > 3) {
		printf("Config channels not correct\n");
		Help();
		return 0;
	}

	options.sampleRate = wavFmt.nSamplesPerSec;

	if(options.quality < 1 || options.quality > 3) {
		Help();
		return 0;
	}

	/* Initilize and alocation */
	if ( miscInitGeneralCallbackTable(&pCallBackTable) != 0 ) {
		return IPP_STATUS_NOMEM_ERR; 
	}

	encoderConfig.allocation = options.allocation;
	encoderConfig.channelMode= options.channels;
	encoderConfig.quality    = options.quality;
	encoderConfig.subbands   = options.subbands;
	encoderConfig.blocks	 = options.blocks;
	encoderConfig.bitpool	 = options.bitpool;

	if (options.quality == 1 || options.quality == 2){
		options.blocks = 16;
		options.subbands = 8;
	}
	sound.sndSampleRate		 = options.sampleRate;
	sound.sndChannelNum = (options.channels > 0)? 2 : 1;
	sound.sndLen = options.blocks * options.subbands * sound.sndChannelNum;

	/* Calculate Length in bytes of one frame */
	join = (options.channels == JOINT_STEREO)? 1 : 0;
	if (options.channels == MONO || options.channels == DUAL_MONO){
		FrameBytes = 4 + ((options.subbands*sound.sndChannelNum<<2)>>3)
			+ ((sound.sndChannelNum*options.blocks*options.bitpool+7)>>3);
	}else{
		FrameBytes = 4 + ((options.subbands*sound.sndChannelNum<<2)>>3)
			+ ((options.subbands*join + options.blocks*options.bitpool+7)>>3);
	}

	/* Max frame bytes of the recommended settings */
	if (options.quality == 1 || options.quality ==2){
		FrameBytes = 119;
	}
	
	sound.pSndFrame = (Ipp16s *)malloc(2 * sound.sndLen);	
	sound.sndSampleRate = options.sampleRate;
	bitstream.pBsBuffer = (Ipp8u *)malloc(FrameBytes); 
	bitstream.bsByteLen = FrameBytes; 
	bitstream.pBsCurByte = bitstream.pBsBuffer;
	bitstream.bsCurBitOffset = 0;

	EncoderInitAlloc_SBC(&pSbcEncoderState, pCallBackTable, &encoderConfig);
	frameLength = sound.sndLen;

	while(frameLength == fread(sound.pSndFrame, sizeof(Ipp16s), frameLength, fpi)){
		Encode_SBC(&sound, &bitstream, pSbcEncoderState);
		fwrite(bitstream.pBsBuffer, 1, bitstream.bsByteLen, fpo);
		++frameNum;
	}

	EncoderFree_SBC(&pSbcEncoderState, pCallBackTable);
	fclose(fpi);
	fclose(fpo);

	free(sound.pSndFrame);
	free(bitstream.pBsBuffer);
	
	miscFreeGeneralCallbackTable(&pCallBackTable);

	return 0;
}

void Help()
{
	printf("SBC encoder of Intel Corp\n");
	printf("Currently we only support the recommended settings in Table 4.7 of the spec!\n");
	printf("Help ----- Command line input:\n");
	printf("exe input output -c [channels] -q [quality]\n");
	printf("	input:			input WAV file name\n");
	printf("	output:			output SBC file name\n");
	printf("	-c [channels]:		channel mode - 0(MONO)|1(DUAL)|2(STEREO)|3(JOINT) \n");
	printf("	-q [quality]:		quality indicator - 1(Middle)|2(high)|3(user define)\n");
	printf("Below parameters need to be configged only when quality==3(user define)\n");
	printf("	-f [sampling]:		sampling frequency - 16000|32000|44100|48000\n");
	printf("	-b [blocks]:		block size - 4|8|12|16");
	printf("	-s [subbands]:		subband in which the stream is encoded -4|8\n");
	printf("	-a [allocation]:	bit allocation method -0(LOUDNESS)|1(SNR)\n");
	printf("	-p [bitpool]:		bitpool - 1~16*nofsubband for mono/dual	- 1~32*nofsubband for stereo/joint\n");
}

void ParseCmdLine(int argc, char **argv, Options *pOptions, FILE** pSrcFile, FILE** pDstFile)
{
	int i;

	 /* Open input file */
	if ((*pSrcFile=fopen(argv[1],"rb"))==NULL)
	{
		printf("Unable to open %s\n",argv[1]);
		exit(1);
	}
	else
		printf("Now Encoding --> %s ",argv[1]);

    /* Open output file */
	if (argc>2)
	{
		if ((*pDstFile=fopen(argv[2],"wb"))==NULL)
		{
			printf("Unable to open %s\n",argv[2]);
			exit(1);
		}
		else
			printf("Output file: %s\n",argv[2]);	
	} else {
		*pDstFile=NULL;
	}

	i = 3;
	while(i < argc) {
		if(!strcmp(argv[i], "-c")) {
			i ++;
			pOptions->channels = atoi(argv[i]);
		} else if(!strcmp(argv[i], "-q")) {
			i ++;
			pOptions->quality = atoi(argv[i]);
		} else if (!strcmp(argv[i], "-f")) {
			i ++;
			pOptions->sampleRate = atoi(argv[i]);		
		} else if (!strcmp(argv[i], "-b")) {
			i ++;
			pOptions->blocks = atoi(argv[i]);
		} else if (!strcmp(argv[i], "-s")) {
			i ++;
			pOptions->subbands = atoi(argv[i]);
		} else if (!strcmp(argv[i], "-a")) {
			i ++;
			pOptions->allocation = atoi(argv[i]);
		} else if (!strcmp(argv[i], "-p")) {
			i++;
			pOptions->bitpool = atoi(argv[i]);
		} 
		i ++;
	}
}
