/******************************************************************************
// INTEL CONFIDENTIAL
// Copyright 2000 ~ 2006 Intel Corporation All Rights Reserved. 
// The source code contained or described herein and all documents related to 
// the source code (Material) are owned by Intel Corporation or its 
// suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material contains trade secrets and 
// proprietary and confidential information of Intel or its suppliers and 
// licensors. The Material is protected by worldwide copyright and trade secret 
// laws and treaty provisions. No part of the Material may be used, copied, 
// reproduced, modified, published, uploaded, posted, transmitted, distributed, 
// or disclosed in any way without Intels prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
******************************************************************************/
#include "misc.h"

#define RESERVED_PARAM(par) (par)=(par)

/**********************************************************/
/* MEMORY CALLBACK IMPLEMENTATIONS */
/**********************************************************/
int miscgMemMalloc(void **ppDstBuf, int size, unsigned char align)
{
    unsigned char   *pAddr = NULL;
    unsigned char   tmp = 0;

    size += align;
    pAddr = malloc(size);
    if (!pAddr) {
        *ppDstBuf = NULL;
        return -1;
    } 
    
    tmp = (unsigned char)((unsigned int)(pAddr) & (align - 1));
    tmp = (unsigned char)(align - tmp);
    pAddr += tmp;
    *(pAddr - 1) = tmp;
    *ppDstBuf = pAddr;
    
    return 0;
}

int miscgMemCalloc(void **ppDstBuf, int size, unsigned char align)
{
    unsigned char   *pAddr = NULL;
    unsigned char   tmp = 0;

    size += align;
    pAddr = calloc(size, 1);
    if (!pAddr) {
        *ppDstBuf = NULL;
        return -1;
    }
    
    tmp = (unsigned char)((unsigned int)(pAddr) & (align - 1));
    tmp = (unsigned char)(align - tmp);
    pAddr += tmp;
    *(pAddr - 1) = tmp;
    *ppDstBuf = pAddr;
    
    return 0;
}

int miscgMemFree(void **ppSrcBuf)
{
    unsigned char   offset = 0;
    unsigned char   *pAddr = NULL;

    pAddr  = (unsigned char*)(*ppSrcBuf);
    offset = *(pAddr - 1);
    pAddr -= offset;
    free((void*)pAddr);

    *ppSrcBuf = NULL;

    return 0;
}

/**********************************************************/
/* STREAM CALLBACK IMPLEMENTATIONS */
/**********************************************************/
int miscgStreamRealloc(void **ppStream, void* pStreamHandler, int oldsize,
					   int newsize)
{
	FILE *fp	  = (FILE*)pStreamHandler;
	void *pStream = *ppStream;	
	RESERVED_PARAM(fp);

	if (newsize > oldsize) {
		pStream   = realloc(pStream, newsize);
		*ppStream = pStream;
		if (!pStream) {
			return -1;
		} else {
			return 0;
		}
	} else {
		return -2;
	}
}

/* return actual flush bytes */
int miscgStreamFlush(void *pStream, void *pStreamHandler, int nAvailBytes, int flag)
{ 
	FILE *fp = (FILE*)pStreamHandler;
    int bytesWritten, total=0;
    int retryTimes = 5;
    RESERVED_PARAM(flag);

    for(;retryTimes>0;retryTimes--) {
        bytesWritten = fwrite((char*)pStream+total, 1, nAvailBytes-total, fp);
        total += bytesWritten;
        if(total>=nAvailBytes) {
            break;
        }
    }
    return total;
}

int miscInitGeneralCallbackTable(MiscGeneralCallbackTable **ppDstCBTable)
{
    MiscGeneralCallbackTable *pTable = NULL;
    
    pTable = (MiscGeneralCallbackTable*)malloc(sizeof(MiscGeneralCallbackTable));
    if (!pTable) {
        printf ("Fails to allocate callback tables!\n");
        return -1;
    }

    pTable->fMemMalloc = (MiscMallocCallback)miscgMemMalloc;
	pTable->fMemCalloc = (MiscCallocCallback)miscgMemCalloc;
    pTable->fMemFree   = (MiscFreeCallback)miscgMemFree;
	pTable->fStreamRealloc = (MiscStreamReallocCallback)miscgStreamRealloc;
    pTable->fStreamFlush   = (MiscStreamFlushCallback)miscgStreamFlush;
	pTable->fFileSeek = (MiscFileSeekCallback)miscSeekFileCallBack;
	pTable->fFileRead = (MiscFileReadCallback)miscReadFileCallBack;
	pTable->fFileWrite = (MiscWriteFileCallBack)miscWriteFileCallBack;
    (*ppDstCBTable) = pTable;
    return 0;
}

int miscFreeGeneralCallbackTable(MiscGeneralCallbackTable **ppSrcCBTable)
{
    MiscGeneralCallbackTable *pTable = (*ppSrcCBTable);
    if (pTable) {
        free (pTable);
        (*ppSrcCBTable) = NULL;
    }

	return 0;
}

int miscSeekFileCallBack(void *handler, int offset, int start)
{
	FILE *file;

	file = (FILE *)handler;

	fseek(file, offset, start);

	return 0;
}

int miscReadFileCallBack(void *pBuf, int unit, int number, void *handler)
{
	unsigned char *pBuffer;
	int readNum;

	pBuffer = (unsigned char *)pBuf;

	readNum = fread(pBuffer, 1, unit * number, (FILE *)handler);

	return readNum;
}

int miscWriteFileCallBack(void *pBuf, int unit, int number, void *handler)
{
	unsigned char *pBuffer;
	int writeNum;

	pBuffer = (unsigned char *)pBuf;

	writeNum = fwrite(pBuffer, 1, unit * number, (FILE *)handler);

	return writeNum;
}
