/******************************************************************************
// INTEL CONFIDENTIAL
// Copyright 2000 ~ 2006 Intel Corporation All Rights Reserved. 
// The source code contained or described herein and all documents related to 
// the source code (Material) are owned by Intel Corporation or its 
// suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material contains trade secrets and 
// proprietary and confidential information of Intel or its suppliers and 
// licensors. The Material is protected by worldwide copyright and trade secret 
// laws and treaty provisions. No part of the Material may be used, copied, 
// reproduced, modified, published, uploaded, posted, transmitted, distributed, 
// or disclosed in any way without Intels prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
******************************************************************************/
#include <stdio.h>
#include <string.h>

#include "codecSC.h"
#include "misc.h"

#define  G729_SAMPLES_PER_FRAME   80
#define BITSTREAM_FILE_FRAME_SIZE 82
/* enum used in example */
typedef	enum {
	RUN_BOTH, RUN_ENCODE, RUN_DECODE
} codec_type;

/* Command line paramter structure */
typedef	struct {
	char *	szFileNameInputPCM;
	char *	szFileNameBitstream;
	char *	szFileNameOutputPCM;
	IppG729CodecType   Mode;	
	codec_type	nCodecType;	
} CmdLineOptions;

extern FILE *mfp; /* message file pointer */

#define MAX_MESSAGE_LEN 256*128
char gMessage[MAX_MESSAGE_LEN];
/* rewrite this OutMessage function for different OS application */
void OutMessage(char *message)
{	
	fprintf(mfp, message);	
	fprintf(mfp, "\n");
}
void Usage()
{	
  char message[1024];

  strcpy(message, "Usage : -i <InputPCMFile> -b <BitstreamFile> -o <OutputPCMFile> \n");
  strcat(message, "   [-c [type]]  specify codec type:  both or encode or decode;\n");
  strcat(message, "   [-m [mode]]  specify codec mode:  G729 or G729A or G729AB;\n");
  strcat(message, "                default is pure G729 codec.\n");
  strcat(message, "   encode 16bit 8000 Hz PCM speech input file\n");
  strcat(message, "   to bitstream output file of format:\n");
  strcat(message, "   81 16bit words per each 80x16bit PCM speech frame:\n");
  strcat(message, "   16bit sync word followed by  \n");
  strcat(message, "   80*16bit words represents 80 decoded bit:\n");
  strcat(message, "   0x007f - represent 0,\n");
  strcat(message, "   0x0081 - represent 1.\n");
  
  OutMessage(message);
}
/*********************************************************************************************
// Name:			 ParseCmdLine
//
// Description:		 Parse commmand line, and set options in CmdLineOptions structure
//
// Input Arguments:  argc			- Number of command line arguments, passed from main
//					 argv			- Content of command line arguments, passed from main
//
// Output Arguments: options		- Pointer to the CmdLineOptions structure, store the options				
//
// Returns:			 Linux style error return code				
**********************************************************************************************/
Ipp32s ParseCmdLine(Ipp32s argc, char ** argv, CmdLineOptions *options)
{
	Ipp32s i = 1;

	Ipp32s err;

	err = 0;	
	options->szFileNameInputPCM = NULL;
	options->szFileNameOutputPCM = NULL;
	options->szFileNameBitstream = NULL;
	options->Mode = G729;	
	while (i < argc) {
		if (!strcmp(argv[i], "-h")) {
			Usage();
		} else if (!strcmp(argv[i], "-i")) {
			options->szFileNameInputPCM = argv[++ i];
		} else if (!strcmp(argv[i], "-o")) {
			options->szFileNameOutputPCM = argv[++ i];
		} else if (!strcmp(argv[i], "-b")) {
			options->szFileNameBitstream = argv[++ i];
		} else if (!strcmp(argv[i], "-m")) {
			i ++;
			if(!strcmp(argv[i], "G729")) {
				options->Mode = G729;
			} else if(!strcmp(argv[i], "G729A")) {
				options->Mode = G729A;
			} else if(!strcmp(argv[i], "G729AB")) {
				options->Mode = G729AB;
			} else {
				OutMessage("codec mode not valid, set to default[G729]\n");
				options->Mode = G729;
			}			
		} else if (!strcmp(argv[i], "-c")) {
			i ++;
			if ( !strcmp(argv[i],"both")) {
				options->nCodecType = RUN_BOTH;
			}
			else if ( !strcmp(argv[i],"encode")) {
				options->nCodecType = RUN_ENCODE;
			}
			else if ( !strcmp(argv[i],"decode")) {
				options->nCodecType = RUN_DECODE;
			}
			else {
				OutMessage("codec state not valid, set to default[both]\n");
				options->nCodecType = RUN_BOTH;
			}
		}

		i ++;
	}

	return 0;
}
/*********************************************************************************************
// Name:			 DisplayOptions
//
// Description:		 display set options
//
**********************************************************************************************/
void DisplayOptions(CmdLineOptions *options)
{
	char message[1024];
	if (options->szFileNameInputPCM != NULL) {
		sprintf(message, "Input PCM file: %s\n",options->szFileNameInputPCM);
		OutMessage(message);
	}
	if (options->szFileNameBitstream != NULL) {
		sprintf(message,  "I/O bitstream file: %s\n",options->szFileNameBitstream);
		OutMessage(message);
	}
	if (options->szFileNameOutputPCM != NULL) {
		sprintf(message,  "Output PCM file: %s\n",options->szFileNameOutputPCM);
		OutMessage(message);
	}	
	
	switch (options->nCodecType) {
	case RUN_BOTH:
		sprintf(message,  "Run Codec Type = Both\n");
		OutMessage(message);
		break;
	case RUN_ENCODE:
		sprintf(message,  "Run Codec Type = Encode\n");
		OutMessage(message);
		break;
	case RUN_DECODE:
		sprintf(message,  "Run Codec Type = Decode\n");
		OutMessage(message);
		break;
	default:
		sprintf(message,  "Run Codec Type = Invalid\n");
		OutMessage(message);
		break;
	}

	switch (options->Mode) {
	case G729:
		sprintf(message,  "Run Codec Mode = G729\n");
		OutMessage(message);
		break;
	case G729A:
		sprintf(message,  "Run Codec Mode = G729A\n");
		OutMessage(message);
		break;
	case G729AB:
		sprintf(message,  "Run Codec Mode = G729AB\n");
		OutMessage(message);
		break;
	default:
		sprintf(message,  "Run Codec Mode = Invalid\n");
		OutMessage(message);
		break;
	}	
}
Ipp32s read_frame(FILE *fBitstream, Ipp16s *buffer, Ipp32s mode){  
	Ipp32s n;
	if (G729 == mode) {
		return(fread(buffer, sizeof(Ipp16s), BITSTREAM_FILE_FRAME_SIZE, fBitstream));
	} else {
		n = fread(buffer, sizeof(Ipp16s), 2, fBitstream);		
		if (n <= 0) return (0);
		fread(buffer + 2, sizeof(Ipp16s), buffer[1], fBitstream);		
		return 1;
	}
}

Ipp32s main(Ipp32s argc, char **argv) 
{
	CmdLineOptions options;
	Ipp32s ret;
	char message[1024];
    
    Ipp32s nFrame=0; Ipp32s sFrame = 366;
    Ipp32s outSize;  
      
    Ipp16s InputBuffer[82], OutputBuffer[82];

    FILE *fInputPCM, *fOutputPCM, *fBitstream;
    void *pState;
	
	MiscGeneralCallbackTable *pCallbackTable = NULL;

	strcpy(gMessage, "");
    ret = ParseCmdLine(argc, argv, &options);
	
	DisplayOptions(&options);
	miscInitGeneralCallbackTable(&pCallbackTable);
	/* Encoder */
	if((options.nCodecType == RUN_ENCODE) || (options.nCodecType == RUN_BOTH)) {
		if ((fInputPCM = fopen(options.szFileNameInputPCM, "rb")) == NULL) {
			sprintf(message, "InputPCM File %s could not be open.\n", options.szFileNameInputPCM);
			OutMessage(message);
			return 1;
		}
		if ((fBitstream = fopen(options.szFileNameBitstream, "wb")) == NULL) {
			sprintf(message, "Bitstream file %s could not be open.\n", options.szFileNameBitstream);
			OutMessage(message);
			return 1;
		}
		if (IPP_STATUS_NOERR != EncoderInitAlloc_G729((void*)&pState, options.Mode, pCallbackTable)) {
			sprintf(message, "Encoder InitAlloc Error!\n");
			OutMessage(message);
			return 1;
		}
		nFrame=0;				
		while(fread(InputBuffer,G729_SAMPLES_PER_FRAME, sizeof(Ipp16s), fInputPCM)){
                  
         nFrame++;
		 if (nFrame >= sFrame) {
			 nFrame = nFrame;			 
		 }
         Encode_G729(pState,InputBuffer,OutputBuffer, options.Mode); /* encode */;
	
         if(G729A == options.Mode || G729AB == options.Mode){            
			outSize = OutputBuffer[1]+2;
         }else {                    
			outSize=BITSTREAM_FILE_FRAME_SIZE;
		 }    

		 fwrite((void*)OutputBuffer, outSize, sizeof(Ipp16s), fBitstream);       		 		
      }   
	  sprintf(message, "%d frame processed.", nFrame);
	  OutMessage(message);
      EncoderFree_G729(&pState, pCallbackTable);	  
	  fclose(fInputPCM);
	  fclose(fBitstream);
	}
   /* Decoder */
	if((options.nCodecType == RUN_DECODE) || (options.nCodecType == RUN_BOTH)) {
		if ((fBitstream = fopen(options.szFileNameBitstream, "rb")) == NULL) {
			sprintf(message, "Bitstream file %s could not be open.\n", options.szFileNameBitstream);
			OutMessage(message);
			return 1;
		}
		if ( (fOutputPCM = fopen(options.szFileNameOutputPCM, "wb")) == NULL) {
			sprintf(message, "OutputPCM File %s could not be open.\n", options.szFileNameInputPCM);
			OutMessage(message);
			return 1;
		}		
		
		if (IPP_STATUS_NOERR != DecoderInitAlloc_G729(&pState, options.Mode, pCallbackTable)) {
			sprintf(message, "Decoder InitAlloc Error!\n");
			OutMessage(message);
			return 1;
		}
		nFrame=0;		
		while(read_frame(fBitstream, InputBuffer, options.Mode)){
			 nFrame++;			 
			 Decode_G729(pState,InputBuffer,OutputBuffer,options.Mode); 
			 fwrite(OutputBuffer, sizeof(Ipp16s), 80, fOutputPCM);
		}
		DecoderFree_G729(&pState, pCallbackTable);
		
		fclose(fOutputPCM);
		fclose(fBitstream);
   }
	miscFreeGeneralCallbackTable(&pCallbackTable);
	OutMessage("End OK");
	
   return 0;
}
