/******************************************************************************
// (C) Copyright 2006 Marvell International Ltd.
// All Rights Reserved
// INTEL CONFIDENTIAL
// Copyright 2000 ~ 2006 Intel Corporation All Rights Reserved. 
// The source code contained or described herein and all documents related to 
// the source code (Material) are owned by Intel Corporation or its 
// suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material contains trade secrets and 
// proprietary and confidential information of Intel or its suppliers and 
// licensors. The Material is protected by worldwide copyright and trade secret 
// laws and treaty provisions. No part of the Material may be used, copied, 
// reproduced, modified, published, uploaded, posted, transmitted, distributed, 
// or disclosed in any way without Intels prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
******************************************************************************/

#ifndef _CODECAC_H_
#define _CODECAC_H_

/* Include Intel IPP external header file(s). */
#include "codecDef.h"	/* General Codec external header file*/
#include "ippAC.h"		/* Audio Codec IPP external header file*/

#ifdef __cplusplus
extern "C" {
#endif

/***** Data Types, Data Structures and Constants ******************************/

/***** Codec-specific Definitions *****/
/* Audio Codecs */
/************************
 AAC Bit Stream Constants
*************************/
#define	IPP_AACD_LAYNUM					8
#define INPUT_BUF_SIZE_AAC			4096
#define OUTPUT_BUF_SIZE_AAC			4096

/************************
 MP3 Bit Stream Constants
*************************/
#define INPUT_BUF_SIZE_MP3				4096
#define OUTPUT_BUF_SIZE_MP3				4608
#define	IPP_MP3_HDR_BUF_SIZE			64   	/* Header buffer size */
#define	IPP_MP3_MAX_GRAN				2		/* Maximum number of granules per channel */
#define	IPP_MP3_MAX_CHAN				2		/* Maximum number of channels */
#define	IPP_MP3_SCF_BANDS				4		/* Number of scalefactor bands per channel */
#define	IPP_MP3_MAIN_DATA_BUF_SIZE		4096	/* Main data decoding buffer */
#define SAMPLE_RATES					3
#define BIT_RATES						15
#define _S_BITMASK(bits)				((1 << bits) - 1)

/* new part for send command */
typedef enum _IppACCommand{
	/* Command for MP3 decoder */
	IPPAC_MP3_GET_SAMPLERATE	 = 0,	/* Get sample rate */
	IPPAC_MP3_GET_BITRATE		 = 1,	/* Get bit rate  */
	IPPAC_MP3_GET_CHANNEL		 = 2,	/* Get channel number */
	IPPAC_MP3_GET_VERSION		 = 3, 	/* Get version */
	IPPAC_MP3_GET_LAYER			 = 4,   /* Get layer infor */
	IPPAC_MP3_GET_SERIES1		 = 5,	/* Get all information from 0~4 */
	IPPAC_MP3_FINDNEXT_SYNCWORD	 = 6,   /* find the next syncword */
	/* Command for AAC Decoder */
	IPPAC_AAC_GET_SAMPLERATE	 = 100,	/* Get sample rate */
	IPPAC_AAC_GET_BITRATE		 = 101,	/* Get bit rate  */
	IPPAC_AAC_GET_CHANNEL		 = 102,	/* Get channel number */
	IPPAC_AAC_GET_VERSION		 = 103, /* Get version */
	IPPAC_AAC_FINDNEXT_SYNCWORD	 = 104,  /* find the next syncword */
	IPPAC_AAC_SET_DOWNSAMPLE_MODE	 = 105	/* Set down sample mode */
} IppACCommand;

typedef struct{
	int version;		/* 0:MPEG1, 1:MPEG2, 2:MPEG2.5 */
	int sampleRate;
	int bitRateIndex;
	int layer;
	int channelNum;
}IppMP3Info1;

typedef struct{
	int mpegFlag;				//2:MPEG-2, 4:MPEG-4
	int audioObjectType;		//2:LC, 4:LTP
	int channelConfiguration;	//output channel configuration, 1:mono, 2:stereo, only support 1 and 2
	int samplingFrequencyIndex;	//0~11, defined in standard 14496-3
	int frameLengthFlag;		//length of frame, 0:1024, 1:960, only support 1024
    int rawFormat;				//only when mpegFlag=4, audioObjectType=2, rawFormat=1 indicate MPEG4-LC raw data.
    /* sbr */
    int sbrEnable;              //SBR tool enable flag, 0: not enable, do not use SBR tool even detect SBR in bit stream, 
                                //                      1: enable, use SBR when detect SBR in bit steam.
	/* ps */
	int psEnable;				//PS tool enable flag, 0: not enable, do not use PS tool even detect PS in bit stream, 
                                //                     1: enable, use PS when detect PS in bit steam.
}IppAACDecoderConfig;

/* SBC Types */
typedef void IppSBCEncoderState;

typedef enum {
	MONO = 0,
	DUAL_MONO,
	STEREO,
	JOINT_STEREO
} IppSBCStreamChannelID;

typedef struct _IppSBCConfig {
	int quality;		/* 0 = low, 1 = middle, 2 = high */
	int subbands;		/* subbands can be 4 or 8 */
	int blocks;
	int bitpool;
	IppSBCStreamChannelID channelMode;	
	int allocation; /* 0 = loudness, 1 = SNR */
} IppSBCConfig;
													
/***** Multimedia Codec Functions *************************************************/
/* MP3 Decoder */
IPPCODECAPI( IppCodecStatus, DecoderInitAlloc_MP3, (
	IppBitstream *pBitStream,
	MiscGeneralCallbackTable *pCallbackTable, 
	IppSound *pDstPcmAudio,
	void **pDecoderState
))

IPPCODECAPI( IppCodecStatus, Decode_MP3, (
    IppBitstream * pSrcBitstream,
    IppSound * pDstPcmAudio,
    void * pDecoderState
))

IPPCODECAPI( IppCodecStatus, DecoderFree_MP3, (
    void ** ppDecoderState
))

/* MPEG-4 AAC LC/LTP, MPEG-2 AAC LC */

IPPCODECAPI( IppCodecStatus, DecoderInitAlloc_AAC, (
	IppBitstream *pBitStream,
	IppAACDecoderConfig *pDecoderConfig,
	MiscGeneralCallbackTable *pCallbackTable, 
	void **ppDecoderState
))


IPPCODECAPI( IppCodecStatus, Decode_AAC, (
	IppBitstream *pBitStream, 
	IppSound *pDstPcmAudio, 
	void *pDecoderState
))

IPPCODECAPI( IppCodecStatus, DecoderFree_AAC,(void **ppDecoderState))

/* Bluetooth SBC */

IPPCODECAPI( IppCodecStatus, EncoderInitAlloc_SBC, (IppSBCEncoderState **ppDs, 
			MiscGeneralCallbackTable* pCallBackTable, 
			IppSBCConfig *pEncoderConfig))

IPPCODECAPI( IppCodecStatus, Encode_SBC, (IppSound *pSrcSound, 
	IppBitstream *pDstBitstream, IppSBCEncoderState *pSBCEncoderState))

IPPCODECAPI( IppCodecStatus, EncoderFree_SBC, (IppSBCEncoderState **ppDs, 
			MiscGeneralCallbackTable* pCallBackTable))

/* New API to get stream informations */
IPPCODECAPI (IppCodecStatus, DecodeSendCmd_MP3Audio, (
    	int	cmd,
		void *pInParam,
    	void *pOutParam,
    	void *pDecoderState
))
IPPCODECAPI (IppCodecStatus, DecodeSendCmd_AACAudio, (
        int  cmd,          
        void *pInParam,       
        void *pOutParam,     
        void *pDecoderState 
))

#ifdef __cplusplus
}
#endif

#endif    /* #ifndef _CODECAC_H_ */

/* EOF */


