/******************************************************************************
// (C) Copyright 2006 Marvell International Ltd.
// All Rights Reserved
******************************************************************************/

#ifndef _IPP_AE_H_
#define _IPP_AE_H_

#include "ippdefs.h"

#ifdef __cplusplus
extern "C" {
#endif

/* macro definition of doppler shift audio */
#define DOPPLER_SOUND_SPEED 21718016		/* 331.4 Q16 format */

/* macro definition of MDRC */
#define MAX_CHANNEL_NUM		2
#define MAX_OCTSTAGE_NUM	10
#define	MAX_SUBOCTSTAGE_NUM	3
#define MAX_BAND_NUM		(MAX_OCTSTAGE_NUM * (MAX_SUBOCTSTAGE_NUM+1) + 1)
#define MAX_MAPPOINTNUM		5

/* macro definition of sample rate conversion */
#define SINC_METHOD 0
#define POLYPHASE_METHOD 1

/* macro definition of graphic equalizer */	
#define MAX_EQ_BAND_NUM 10


/******************************************************************************
// External structures
******************************************************************************/

/* State structure name definition */
typedef void IppAECState;
typedef void IppStereoWidenState;
typedef void IppMDRCState;
typedef void IppBassBoostState;
typedef void IppEqualizerState;
typedef void IppAudioMixerState;
typedef void IppResampleState;
typedef void IppHRTFState;
typedef void IppDopplerState;
typedef void IppEQBBSWState;

/* Ipp Sound Structure */
typedef struct _IppSound {
    Ipp16s *    pSndFrame;
    int         sndLen;
    int         sndChannelNum;
    int         sndSampleRate;
} IppSound;

/* 3D positional audio: sound emitter model */

#define HRTF_ATTEN_MAX_DB -20				/* Max attenuation in outer ellipse in dB */

typedef struct _IppEmitterModel {

	int iInnerRadius;						/* Radius of the inner circle */
	int iOuterRadius;						/* Radius of the outer circle */
	int iAttenMax_dB;						/* Max attenuation in outer ellipse in dB */
	
}IppEmitterModel;


/* Acoustic echo cancellation structure */

typedef enum {
    IPP_PCM_FREQ_8000 = 8000,                /* 8  kHz  */
    IPP_PCM_FREQ_16000 = 16000               /* 16 kHz */
} IppPCMFrequency;

typedef enum aec_mode_enum {
    EC_COEFFS_ZERO = 0,
    EC_ADAPTATION_ENABLE,
    EC_ADAPTATION_DISABLE,
    EC_NLP_ENABLE,
    EC_NLP_DISABLE,
} AECModeCode;

/* Stereo widening structure */


/* Multiband DRC structure */
typedef struct _IppMDRCConfig {
	/* Basic settings */
	int iChannelNum;
	int iSampleRate;

	/* Sub-band settings */
	Ipp32s iBandOverlapQ16;					/* ratio between transition-band width between subbands and subband-width */
	int iLPCutOffFreq;						/* cut-off freq of the lowest subband */
	int iOctStageNum;						/* number of octaves, decide the freq coverage */
	int iSubOctStageNum;					/* number of sub-octaves resolution, decide the freq resolution. */

	/* Dynamic law settings */
	int iLookAheadTime;						/* used to apply gain change ahead of abrupt impulse, FOR FUTURE EXPANSION */
	int bRMSLevelDetectMode;				/* Level detection mode: RMS or peak */
	int bAutoReleaseControl;				/* Enable this to control the release time automatically */

	Ipp32s LATQ16[MAX_BAND_NUM];			/* attack time for level detector, in milliseconds */
	Ipp32s LRTQ16[MAX_BAND_NUM];			/* release time for level detector, in milliseconds */
	Ipp32s GATQ16[MAX_BAND_NUM];			/* attack time for gain smoothing */
	Ipp32s GRTQ16[MAX_BAND_NUM];			/* release time for gain smoothing */

	/* Static law basic settings */
	int iMapPointNum;						/* Point number in the in-out mapping curve */
	Ipp32s knee;							/* used to control the smoothness of curve turning points, FOR FUTURE EXPANSION */
	Ipp32s iOutGainQ16;						/* Extra overall gain for final output */

	Ipp32s iMappingCurveQ16[MAX_BAND_NUM][MAX_MAPPOINTNUM*2];	/* Mapping curves for each subband */
} IppMDRCConfig;



/* Fixed audio mixer structure */



/* Sample rate conversion structure */
typedef enum {
	DEFAULT,
	POLYPHASE,
	SINC,
	RESERVED
}SRC_SCHEME_DEF;

typedef struct {
	int NumIn;
	int NumOut;
	int sincBufSize;
	int polyBufSize;
}bufInfo;

typedef struct {
	int inSampleRate;
	int outSampleRate;
	int channel;
	int NumIn;
	SRC_SCHEME_DEF scheme;
	void *pUsrParameter;
}IppSRCConfig;


/* Bass boost structure */
typedef struct {

	int iSampleRate;						/* Input audio sample rate */
	int iChannelNum;						/* Input channel number */
	int iLowFreq;							/* Low bound freq to be boosted in Hz */
	int iHighFreq;							/* High bound freq to be boosted in Hz */
	int iGain_dB;							/* Gain for LFE in dB */
	int iBoostMethod;						/* Bass boost method. 1: LFE, others: reserved */
	int iWinType;							/* Window type. 1: Hanning, others: reserved */

}IppBassBoostConfig;



/* Graphic audio equalizer structure */
typedef struct {

	int iSampleRate;						/* Input audio sample rate */
	int iChannelNum;						/* Input channel number */
	int iBandNum;							/* Band number for equalizer */
	int iUseDefaultPartition;				/* Use default band partition method. Only valid for 3, 5 and 10  band */
	int iCentFreq[MAX_EQ_BAND_NUM];
	int pGain_dB[MAX_EQ_BAND_NUM];
	int iEnableBB;							/* Enable bass boost or not */
	int iLowFreq;							/* Low bound freq to be boosted in Hz */
	int iHighFreq;							/* High bound freq to be boosted in Hz */
	int iBBGain_dB;							/* Gain for bass boost in dB */

}IppEqualizerConfig;


/******************************************************************************
// External functions
******************************************************************************/

/******************************************************************************
// 3D positional audio API
******************************************************************************/

IPPAPI(IppStatus, ippsHRTF_GetSize, (int *pSize))

IPPAPI(IppStatus, ippsHRTF_Init, (void *pEmitterModel, 
	   void *pfCallbackCalGain, IppHRTFState *pHRTFState))

IPPAPI(IppStatus, ippsHRTF, (const IppSound *pSrc, IppSound *pDst, Ipp32s iGainQ16,
	   int azimuth, int elevation, int distance, void *pReserved, IppHRTFState *pHRTFState))

IPPAPI(IppStatus, ippsHRTF_CalGain, (void *pModel, int az, int el, int distance, void *pReserved, 
	   Ipp32s *pDistanceGainQ16))

/******************************************************************************
// Doppler shift audio API
******************************************************************************/

IPPAPI(IppStatus, ippsDoppler_GetSize, (int iOutSampleCount, int *pSize))

IPPAPI(IppStatus, ippsDoppler_Init, (int iOutSampleCount, Ipp32s pitchCorrection, Ipp32s SoundSpeed, void *pDopplerState))

IPPAPI(IppStatus, ippsDoppler_CalInputSamples, (Ipp32s SourceSpeed, Ipp32s ListenerSpeed, int outSampleCount, int *inSampleCount, IppDopplerState *pDopplerState))

IPPAPI(IppStatus, ippsDoppler, (const IppSound *pSrc, IppSound *pDst, IppDopplerState *pDopplerState))


/******************************************************************************
// Acoustic Echo Cancellation API
******************************************************************************/

IPPAPI(IppStatus, ippsAEC_GetSize, (IppPCMFrequency freq, int taptime_ms, int *size))

IPPAPI(IppStatus, ippsAEC_Init, (IppAECState *pState, IppPCMFrequency freq, int taptime_ms))

IPPAPI(IppStatus, ippsAEC, (IppAECState *pState, const IppSound *pRin, const IppSound *pSin, IppSound *pSout))

IPPAPI(IppStatus, ippsAEC_GetFrameSize, (IppPCMFrequency freq, int taptime_ms, int *s))

IPPAPI(IppStatus, ippsAEC_SetMode, (IppAECState *pState, AECModeCode op))

/******************************************************************************
// Stereo widening API
******************************************************************************/

IPPAPI(IppStatus, ippsStereoWidening_GetSize, (int* pStateSize))

IPPAPI(IppStatus, ippsStereoWidening_Init, (int iSampleRate, int realAngle, 
	   int realElev, int phantomAngle, int phantomElev,
	   Ipp32s iGainQ16, int bEnableCTC, IppStereoWidenState* pState))

IPPAPI(IppStatus, ippsStereoWidening, (const IppSound* pSrc, IppSound* pDst, 
	   IppStereoWidenState* pState))


/******************************************************************************
// Multiband DRC API
******************************************************************************/

IPPAPI(IppStatus, ippsMDRC_GetSize, (const IppMDRCConfig* pMDRCConfig, 
	   int *pFrameLen, int *pStateSize))

IPPAPI(IppStatus, ippsMDRC_Init, (
	   const IppMDRCConfig* pMDRCConfig, IppMDRCState *pState))

IPPAPI(IppStatus, ippsMDRC, (
	   const IppSound* pSrc, IppSound* pDst, IppMDRCState *pState))


/******************************************************************************
// Audio Mixing API
******************************************************************************/

IPPAPI(IppStatus, ippsMixer_GetSize, (const IppSound* pSrc, int iSrcNum,
	   IppSound* pDst, int iDstNum, int *pBufSize))

IPPAPI(IppStatus, ippsMixer_Init, (const IppSound* pSrc, int iSrcNum,
	   IppSound* pDst, int iDstNum, Ipp32s *pGain, /*int* pDelay, */IppAudioMixerState *pState))

/* Ipp32s *pGain should be put into mix */
IPPAPI(IppStatus, ippsMixer, (const IppSound* pSrc, IppSound* pDst, int* pDstMaxLen,
	   int *pUsedNum, IppAudioMixerState *pState))


/******************************************************************************
// Sample Rate Conversion
******************************************************************************/

IPPAPI(IppStatus, ippsResample_GetSize,(IppSRCConfig *pConfig, 
	   bufInfo *pBufInfo, int *bufSize))

IPPAPI(IppStatus, ippsResample_Init,(IppSRCConfig *pConfig, 
	   IppResampleState *pResampleState, bufInfo *pBufInfo, Ipp8s *pBuffer))

IPPAPI(IppStatus, ippsResample,(IppResampleState *pResampleState, const IppSound *pInSound, 
	   IppSound *pOutSound))

/******************************************************************************
// Bass Boost
******************************************************************************/

IPPAPI(IppStatus, ippsBassBoost_GetSize, (int *pSize))

IPPAPI(IppStatus, ippsBassBoost_Init, (IppBassBoostConfig *pConfig, 
	   IppBassBoostState *pBassBoostState))

IPPAPI(IppStatus, ippsBassBoost, (const IppSound *pSrc, IppSound *pDst, 
	   IppBassBoostState *pBassBoostState))


/******************************************************************************
// Graphic Audio Equalizer
******************************************************************************/

IPPAPI(IppStatus, ippsEqualizer_GetSize, (IppEqualizerConfig *pConfig, int *pSize))

IPPAPI(IppStatus, ippsEqualizer_Init, (IppEqualizerConfig *pConfig, 
	   IppEqualizerState *pEqualizerState))

IPPAPI(IppStatus, ippsEqualizer, (const IppSound *pSrc, IppSound *pDst, 
	   Ipp32s iGainQ16, IppEqualizerState *pEqualizerState))



#ifdef __cplusplus                       
}                                        
#endif

#endif // _IPP_AE_H_
