/******************************************************************************
// (C) Copyright 2007 Marvell International Ltd.
// All Rights Reserved
// INTEL CONFIDENTIAL
// Copyright 2000 ~ 2006 Intel Corporation All Rights Reserved. 
// The source code contained or described herein and all documents related to 
// the source code (Material) are owned by Intel Corporation or its 
// suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material contains trade secrets and 
// proprietary and confidential information of Intel or its suppliers and 
// licensors. The Material is protected by worldwide copyright and trade secret 
// laws and treaty provisions. No part of the Material may be used, copied, 
// reproduced, modified, published, uploaded, posted, transmitted, distributed, 
// or disclosed in any way without Intels prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
******************************************************************************/

#ifndef _IPPSP_H_
#define _IPPSP_H_

#include "ippdefs.h"

#ifdef __cplusplus
extern "C" {
#endif

#define IPP_MAX_EXP 308

#ifndef _CALLBACK
#define _CALLBACK __STDCALL
#endif

void* _CALLBACK ippsMalloc(int size);
void  _CALLBACK ippsFree(void *pSrcBuf);

/* =============== Structure Definition for 2^N variant length FFT ============== */

typedef void IppsFFTSpec_C_16sc;
typedef void IppsFFTSpec_C_32sc;
typedef void IppsFFTSpec_R_16s32s;
typedef void IppsFFTSpec_R_32s;

/* =========================== Filter Functions ============================ */
	
/* FIR single rate */
IPPAPI(IppStatus, ippsFIR_Direct_16s,	
	(const Ipp16s * pSrc, Ipp16s * pDst, int sampLen, const Ipp16s * pTapsQ15, 
	int tapsLen, Ipp16s * pDelayLine, int * pDelayLineIndex))

IPPAPI(IppStatus, ippsFIR_Direct_16s_I,	
	(Ipp16s * pSrcDst, int sampLen, const Ipp16s * pTapsQ15, int tapsLen, 
	Ipp16s * pDelayLine, int * pDelayLineIndex))

IPPAPI(IppStatus, ippsFIROne_Direct_16s,
	(Ipp16s val, Ipp16s * pResult, const Ipp16s * pTapsQ15, int tapsLen, 
	Ipp16s * pDelayLine, int * pDelayLineIndex))

IPPAPI(IppStatus, ippsFIROne_Direct_16s_I,	
	(Ipp16s * pValResult, const Ipp16s * pTapsQ15, int tapsLen, 
	Ipp16s * pDelayLine, int * pDelayLineIndex))


IPPAPI(IppStatus, ippsFIR_Direct_16s_Sfs,	
	(const Ipp16s * pSrc, Ipp16s * pDst, int sampLen, const Ipp16s * pTapsQ15, 
	int tapsLen, Ipp16s * pDelayLine, int * pDelayLineIndex, int scaleFactor))

IPPAPI(IppStatus, ippsFIR_Direct_16s_ISfs,	
	(Ipp16s * pSrcDst, int sampLen, const Ipp16s * pTapsQ15, int tapsLen, 
	Ipp16s * pDelayLine, int * pDelayLineIndex, int scaleFactor))

IPPAPI(IppStatus, ippsFIROne_Direct_16s_Sfs,
	(Ipp16s val, Ipp16s * pResult, const Ipp16s * pTapsQ15, int tapsLen, 
	Ipp16s * pDelayLine, int * pDelayLineIndex, int scaleFactor))

IPPAPI(IppStatus, ippsFIROne_Direct_16s_ISfs,	
	(Ipp16s * pValResult, const Ipp16s * pTapsQ15, int tapsLen, 
	Ipp16s * pDelayLine, int * pDelayLineIndex, int scaleFactor))

/* FIR Multirate */
IPPAPI(IppStatus, ippsFIRMR32s_Direct_16s_Sfs,
	(const Ipp16s*	pSrc, Ipp16s* pDst, int numIters, const Ipp32s*	pTaps,
	int tapsLen, int tapsFactor, int upFactor, int upPhase, int downFactor,
	int downPhase, Ipp16s* pDlyLine, int scaleFactor))

/* IIR direct form */
IPPAPI(IppStatus, ippsIIR_Direct_16s,	
	(const Ipp16s * pSrc, Ipp16s * pDst, int len, const Ipp16s * pTaps, 
	int order, Ipp32s * pDelayLine))

IPPAPI(IppStatus, ippsIIR_Direct_16s_I,	
	(Ipp16s * pSrcDst, int len, const Ipp16s * pTaps, int order, 
	Ipp32s * pDelayLine))

IPPAPI(IppStatus, ippsIIROne_Direct_16s,
	(Ipp16s val, Ipp16s * pResult, const Ipp16s * pTaps, int order, 
	Ipp32s * pDelayLine))

IPPAPI(IppStatus, ippsIIROne_Direct_16s_I,
	(Ipp16s * pValResult, const Ipp16s * pTaps, int order, Ipp32s * pDelayLine))

/* IIR biquad */
IPPAPI(IppStatus, ippsIIR_BiQuadDirect_16s,	
	(const Ipp16s * pSrc, Ipp16s * pDst, int len, const Ipp16s * pTaps, 
	int numBiquad, Ipp32s * pDelayLine))

IPPAPI(IppStatus, ippsIIR_BiQuadDirect_16s_I,
	(Ipp16s * pSrcDst, int len, const Ipp16s * pTaps, int numBiquad, 
	Ipp32s * pDelayLine))

IPPAPI(IppStatus, ippsIIROne_BiQuadDirect_16s,
	(Ipp16s val, Ipp16s * pResult, const Ipp16s * pTaps, int numBiquad, 
	Ipp32s * pDelayLine))

IPPAPI(IppStatus, ippsIIROne_BiQuadDirect_16s_I,
	(Ipp16s * pValResult, const Ipp16s * pTaps, int numBiquad, 
	Ipp32s * pDelayLine))

/* LMS FIR */
IPPAPI(IppStatus, ippsFIRLMSOne_Direct_16s,	
	(Ipp16s val, Ipp16s valDesire, Ipp16s * pResult, Ipp32s * pTapsQ15, 
	int tapsLen, Ipp32s stepQ15, Ipp16s * pDelayLine, int * pDelayLineIndex))

IPPAPI(IppStatus, ippsFIRLMSOne_Direct_16s_I,	
	(Ipp16s * pValResult, Ipp16s valDesire, Ipp32s * pTapsQ15, int tapsLen, 
	Ipp32s stepQ15, Ipp16s * pDelayLine, int * pDelayLineIndex))
		
/* ========================== Transform Functions ========================== */

/* Variant-length (2^order) 16-bit complex FFT */
IPPAPI(IppStatus, ippsFFTFwd_CToC_16sc_Sfs,
    (const Ipp16sc *pSrc, Ipp16sc *pDst, const IppsFFTSpec_C_16sc *pFFTSpec, 
    int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTInv_CToC_16sc_Sfs,
	(const Ipp16sc *pSrc, Ipp16sc *pDst, const IppsFFTSpec_C_16sc *pFFTSpec,
	 int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTInitAlloc_C_16sc,
    (IppsFFTSpec_C_16sc** pFFTSpec, int order, int flag, IppHintAlgorithm hint))

IPPAPI(IppStatus, ippsFFTGetBufSize_C_16sc,
	(const IppsFFTSpec_C_16sc *pFFTSpec, int *pSize))

IPPAPI(IppStatus, ippsFFTFree_C_16sc,
    (IppsFFTSpec_C_16sc* pFFTSpec))
    
    
/* Variant-length (2^order) 32-bit complex FFT */
IPPAPI(IppStatus, ippsFFTFwd_CToC_32sc_Sfs,
    (const Ipp32sc *pSrc, Ipp32sc *pDst, const IppsFFTSpec_C_32sc *pFFTSpec,
	int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTInv_CToC_32sc_Sfs,
    (const Ipp32sc *pSrc, Ipp32sc *pDst, const IppsFFTSpec_C_32sc *pFFTSpec,
	int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTInitAlloc_C_32sc,
    (IppsFFTSpec_C_32sc** pFFTSpec, int order, int flag, IppHintAlgorithm hint))

IPPAPI(IppStatus, ippsFFTGetBufSize_C_32sc,
	(const IppsFFTSpec_C_32sc *pFFTSpec, int *pSize))

IPPAPI(IppStatus, ippsFFTFree_C_32sc,
    (IppsFFTSpec_C_32sc* pFFTSpec))
    

/* Variant-length (2^order) real->complex FFT */
IPPAPI(IppStatus, ippsFFTFwd_RToCCS_16s32s_Sfs,
    (const Ipp16s *pSrc, Ipp32s *pDst, const IppsFFTSpec_R_16s32s *pFFTSpec,
	int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTInv_CCSToR_32s16s_Sfs,
    (const Ipp32s *pSrc, Ipp16s *pDst, const IppsFFTSpec_R_16s32s *pFFTSpec,
	int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTFwd_RToCCS_32s_Sfs,
    (const Ipp32s *pSrc, Ipp32s *pDst, const IppsFFTSpec_R_32s *pFFTSpec,
	int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTInv_CCSToR_32s_Sfs,
    (const Ipp32s *pSrc, Ipp32s *pDst, const IppsFFTSpec_R_32s *pFFTSpec,
	int scaleFactor, Ipp8u *pBuffer))

IPPAPI(IppStatus, ippsFFTInitAlloc_R_16s32s,
    (IppsFFTSpec_R_16s32s** pFFTSpec, int order, int flag, 
	IppHintAlgorithm hint))

IPPAPI(IppStatus, ippsFFTInitAlloc_R_32s,
    (IppsFFTSpec_R_32s** pFFTSpec, int order, int flag, 
	IppHintAlgorithm hint))

IPPAPI(IppStatus, ippsFFTGetBufSize_R_16s32s,
	(const IppsFFTSpec_R_16s32s *pFFTSpec, int *pSize))

IPPAPI(IppStatus, ippsFFTGetBufSize_R_32s,
	(const IppsFFTSpec_R_32s *pFFTSpec, int *pSize))

IPPAPI(IppStatus, ippsFFTFree_R_16s32s,
    (IppsFFTSpec_R_16s32s* pFFTSpec))

IPPAPI(IppStatus, ippsFFTFree_R_32s,
    (IppsFFTSpec_R_32s* pFFTSpec))    


#ifdef __cplusplus
}
#endif

#endif /* _IPPSP_H_ */ 
	
/* End of file */
