//***************************************************************************
//
// (C) Copyright [2007-2-5] Marvell International Ltd.  
// All Rights Reserved  
//
// ******************************************************************************
#include <windows.h>
#include <GLES/egl.h>
#include <GLES/gl.h>

#define USE_PIXMAP 1

// Global Variables:
HINSTANCE	g_hInst = NULL;                // The current instance
HWND		g_hWnd	= NULL;
static EGLDisplay		eglDisplay			= NULL;
static EGLConfig		eglConfig_W			= NULL;
static EGLConfig		eglConfig_P			= NULL;
static EGLContext		eglContext_W		= NULL;
static EGLContext		eglContext_P		= NULL;
static EGLSurface		eglPixmapSurface	= NULL;
static EGLSurface		eglWindowSurface	= NULL;

// The attibutes required to match by the configurations of windowsurface
static const EGLint s_configAttribs_W[] =
{
    EGL_RED_SIZE,       5,
		EGL_GREEN_SIZE,     6,
		EGL_BLUE_SIZE,      5,
		
		EGL_ALPHA_SIZE,     EGL_DONT_CARE,
		EGL_DEPTH_SIZE,     16,
		EGL_STENCIL_SIZE,   EGL_DONT_CARE,
		EGL_SURFACE_TYPE,   EGL_WINDOW_BIT,
		EGL_NONE
};

// The attibutes required to match by the configurations of pixmapsurface
static const EGLint s_configAttribs_P[] =
{
    EGL_RED_SIZE,       5,
		EGL_GREEN_SIZE,     6,
		EGL_BLUE_SIZE,      5,
		
		EGL_ALPHA_SIZE,     EGL_DONT_CARE,
		EGL_DEPTH_SIZE,     16,
		EGL_STENCIL_SIZE,   EGL_DONT_CARE,
		EGL_SURFACE_TYPE,   EGL_PIXMAP_BIT,
		EGL_NONE
};

int Width=240, Height=320;

static int mode = 0;

NWSPixmap		local_pixmap;
NWSPixmap*		pNWSpixmap = &local_pixmap;

GLfixed v[] = 
{
        32768, 0, 0,
        0, 0, 0,
        0, 32768, 0
};

// Fog color
GLfixed fog_Color[4] = {19660, 19660, 19660, 32768};
static HBITMAP  pixmap;
DWORD*          bits;
BITMAPINFO*     bmi;

//*****************************************************************************
// Name:            CreatePixmap
// Description:     Application initialization 
//*****************************************************************************
static HBITMAP CreatePixmap (
							 HDC hDC)
{
		// Define the HBitmap Header info
    const size_t    bmiSize = sizeof(BITMAPINFO) + 256U*sizeof(RGBQUAD);
    DWORD*          p;
    bmi = (BITMAPINFO*)malloc(bmiSize);
    memset(bmi, 0, bmiSize);
	
    bmi->bmiHeader.biSize           = sizeof(BITMAPINFOHEADER);
    bmi->bmiHeader.biWidth          = Width,
		bmi->bmiHeader.biHeight         = Height,
		
		bmi->bmiHeader.biPlanes         = (short)1;
    bmi->bmiHeader.biBitCount       = (unsigned int) 16;
    bmi->bmiHeader.biCompression    = BI_BITFIELDS;
    bmi->bmiHeader.biSizeImage      = 0;
    bmi->bmiHeader.biXPelsPerMeter  = 0;
    bmi->bmiHeader.biYPelsPerMeter  = 0;
    bmi->bmiHeader.biClrUsed        = 3;
    bmi->bmiHeader.biClrImportant   = 0;
		// The color buffer will be R5G6B5 format
    p = (DWORD*)bmi->bmiColors;
    p[0] = 0x1F << 11;   /* r */
    p[1] = 0x3F << 5;   /* g */
    p[2] = 0x1F << 0;  /* b */
	
		// Create a HBitmap whose color buffer will be used as the pixmap surface
    return  CreateDIBSection(hDC, bmi, DIB_RGB_COLORS, (void **)&bits, NULL, 0);
}


static void deletePixmap (
						  HBITMAP bitmap)
{
    DeleteObject(bitmap);
}




//*****************************************************************************
// Name:            AppInit
// Description:     Application initialization 
//*****************************************************************************
BOOL AppInit( void )
{
    EGLint      numConfigs;
    EGLint      majorVersion;
    EGLint      minorVersion;
	
	// Create pixmap
	HDC dc = GetDC(g_hWnd);
	pixmap = CreatePixmap(dc);
	
	// Wrapper the HBitmap to a Native Window System Pixmapsurface
	pNWSpixmap->pBits = bits;			// Share the same color buffer with the HBitmap
	pNWSpixmap->iWidth = Width;
	pNWSpixmap->iHeight = Height;
	pNWSpixmap->iStride = Width * 2;
	pNWSpixmap->eFormat = NWSFMT_R5G6B5;	// The same color format as the HBitmap
	ReleaseDC(g_hWnd, dc);
	
	// Get EGL display
    eglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    if(eglDisplay == EGL_NO_DISPLAY || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }
	
    // Initialize EGL
    eglInitialize(eglDisplay, &majorVersion, &minorVersion);
    if(eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }
	
    // Choose EGL config
	if (!eglChooseConfig(eglDisplay, s_configAttribs_W, &eglConfig_W, 1, &numConfigs))
	{
		printf("error choosing window configs \n");
		return FALSE;
	}
	if(!eglChooseConfig(eglDisplay, s_configAttribs_P, &eglConfig_P, 1, &numConfigs))
	{
		printf("error choosing pixmap configs \n");
		return FALSE;
	}
	
    // Create EGL rendering context
    eglContext_W = eglCreateContext(eglDisplay, eglConfig_W, NULL, NULL);
    if(eglContext_W == EGL_NO_CONTEXT || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }
    eglContext_P = eglCreateContext(eglDisplay, eglConfig_P, NULL, NULL);
    if(eglContext_P == EGL_NO_CONTEXT || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }
	
    // Create PixmapSurface 
    eglPixmapSurface = eglCreatePixmapSurface(eglDisplay, eglConfig_P, pNWSpixmap, NULL);
    if(eglPixmapSurface == EGL_NO_SURFACE || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }
    
    //Create WindowSurface for eglSwapBuffer() to render
		eglWindowSurface = eglCreateWindowSurface(eglDisplay, eglConfig_W, g_hWnd, NULL);
		if(eglWindowSurface == EGL_NO_SURFACE || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }	
	
	if(USE_PIXMAP)
	{
		// Attach the EGL rendering context to EGL surfaces
		eglMakeCurrent(eglDisplay, eglPixmapSurface, eglPixmapSurface, eglContext_P);
		if(eglGetError() != EGL_SUCCESS)
		{
			return FALSE;
		}
	}
	else
	{
		// Attach the EGL rendering context to EGL surfaces
		eglMakeCurrent(eglDisplay, eglWindowSurface, eglWindowSurface, eglContext_W);
		if(eglGetError() != EGL_SUCCESS)
		{
			return FALSE;
		}		
	}
	
    return TRUE;
}



//*****************************************************************************
// Name:            AppDeInit
// Description:     Application release 
//*****************************************************************************
void AppDeInit( void )
{
    // Destroy all EGL resources    
    eglMakeCurrent(eglDisplay, NULL, NULL, NULL);
    eglDestroyContext(eglDisplay, eglContext_P);
    eglDestroySurface(eglDisplay, eglPixmapSurface);
    eglDestroyContext(eglDisplay, eglContext_W);
    eglDestroySurface(eglDisplay, eglWindowSurface);
    eglTerminate(eglDisplay);
	
	deletePixmap(pixmap);
    return;
}



//*****************************************************************************
// Name:            AppUpdate
// Description:     Application update 
//*****************************************************************************
void AppUpdate( void )
{
    // Change blending mode
    mode ++;
	
	
	// Enable blending
    glEnable(GL_BLEND);
	
	// Set the pixel arithmetic operation used by blending
    switch(mode%4)
    {
    case 0:
        glColor4x(0, 65536, 65536, 65536);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        break;
    case 1:
        glColor4x(0, 0, 65536, 26214);
        glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
        break;
    case 2:
        glColor4x(0, 32768, 32768, 6553);
        glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
        break;
    case 3:
        glDisable(GL_BLEND);
        break;
    }
	
    return;
}


void SwapBuffers( void )
{
	if(USE_PIXMAP)
	{	
		InvalidateRect(g_hWnd, NULL, FALSE);		
	}
	else
	{
		// Swap the color buffer to window surface 
		eglSwapBuffers(eglDisplay, eglWindowSurface);
	}
	return;	
}

//*****************************************************************************
// Name:            AppRender
// Description:     Application rendering. Three triangles will be rendered abreast and their color 
//                  change according to different blending function setting.   
//*****************************************************************************
void AppRender( void )
{
	// Clear the color buffer with red
    glClearColorx(65536, 0, 0, 0);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	
	// Set vertex pointer and enable the vertex array for rendering
    glVertexPointer(3, GL_FIXED, 0, v);
    glEnableClientState( GL_VERTEX_ARRAY);
	
	// Set projection matrix
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustumx(-131072, 131072, -65536, 65536, 65536, 458752);
	
	// Set model-view matrix
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
	
	// Draw the triangle for three times
    // For each time a different translation is set
    glPushMatrix();
    glTranslatex(-65536, 0, -65536);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glPopMatrix();
	
    glPushMatrix();
    glTranslatex(0, 0, -131072);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glPopMatrix();
	
    glPushMatrix();
    glTranslatex(65536, 0, -196608);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glPopMatrix();
	
    SwapBuffers();
	
	// Sleep for 500 ms
    Sleep(500);
    return;
}



//*****************************************************************************
// Name:            WndProc
// Description:     Processes messages for the main window 
//*****************************************************************************
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) 
    {
#if USE_PIXMAP		
	case WM_PAINT:
		{		
			// Since pixmapsurface share the same color buffer with the HBitmap,
			// after rendering the HBitmap, the result of OpenGL ES pipeline will appear
			PAINTSTRUCT     ps;
			HDC dc = BeginPaint(g_hWnd, &ps);
			HDC hdcBitmap = CreateCompatibleDC((HDC)dc);
			HDC old = (HDC)SelectObject(hdcBitmap, pixmap);
			
			
			int dx1 = GetSystemMetrics(SM_CXSCREEN);
			int dy1 = GetSystemMetrics(SM_CYSCREEN);
			
			StretchBlt((HDC)dc, 0, 0, dx1, dy1, hdcBitmap, 0, 0, dx1, dy1, SRCCOPY);
			
			SelectObject(hdcBitmap, old);
			
			EndPaint(g_hWnd, &ps);
			break;		
		}
#endif		
	case WM_KEYDOWN:
	case WM_LBUTTONDOWN:
	case WM_CLOSE:
	case WM_CANCELMODE:
		DestroyWindow (hWnd);
		break;
		
	case WM_DESTROY:
		PostQuitMessage (0);
		break;
		
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
    }
	
    return 0;
}

//*****************************************************************************
// Name:            MyRegisterClass
// Description:     Registers the window class 
//*****************************************************************************
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
    WNDCLASS    wc;
	
    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc      = (WNDPROC) WndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = hInstance;
    wc.hIcon            = NULL;
    wc.hCursor          = 0;
    wc.hbrBackground    = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName     = 0;
    wc.lpszClassName    = szWindowClass;
	
    return RegisterClass(&wc);
}



//*****************************************************************************
// Name:            InitInstance
// Description:     Save application instance handle and creates main window 
//*****************************************************************************
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{    
	
    // Store instance handle in our global variable
    g_hInst = hInstance;        
	
    //If it is already running, then focus on the window
    g_hWnd = FindWindow(L"OGL-ES Sample", L"OGL-ES Sample");  
    if (g_hWnd) 
    {
        // set focus to foremost child window
        // The "| 0x01" is used to bring any owned windows to the foreground and
        // activate them.
        SetForegroundWindow((HWND)((ULONG) g_hWnd | 0x00000001));
        return 0;
    }
	
    // Register window class
    MyRegisterClass(hInstance, L"OGL-ES Sample");
	
    // Create main application window
    g_hWnd = CreateWindow( L"OGL-ES Sample", L"OGL-ES Sample", WS_VISIBLE,
		0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), 
		NULL, NULL, hInstance, NULL );
    if (!g_hWnd)
    {   
        return FALSE;
    }
	
    // Show main window
    ShowWindow(g_hWnd, nCmdShow);
    UpdateWindow(g_hWnd);    
	
    return TRUE;
}



//*****************************************************************************
// Name:            WinMain
// Description:     Application entry 
//*****************************************************************************
int WINAPI WinMain (
					HINSTANCE   hInstance, 
					HINSTANCE   hPrevInstance, 
					LPWSTR      lpCmdLine, 
					int         nCmdShow)
{
    MSG msg;
	
    // Perform application initialization:
    if( !InitInstance (hInstance, nCmdShow) ) 
    {
        return FALSE;
    }
	
    // Application initialization
    if( !AppInit() )
    {
        return FALSE;
    }
	
    while( TRUE )
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
        {
            if( WM_QUIT == msg.message )
                break;
            TranslateMessage( &msg );
            DispatchMessage ( &msg );
        }
        else
        {
            // Application update
            AppUpdate();
			
            // Application render
            AppRender();
        }
    }
	
	AppDeInit();
    return msg.wParam;
}


// EOF