/******************************************************************************
// (C) Copyright 2006 Marvell International Ltd.
// All Rights Reserved
// INTEL CONFIDENTIAL
// Copyright 2004 ~ 2006 Intel Corporation All Rights Reserved. 
// The source code contained or described herein and all documents related to 
// the source code (Material) are owned by Intel Corporation or its 
// suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material contains trade secrets and 
// proprietary and confidential information of Intel or its suppliers and 
// licensors. The Material is protected by worldwide copyright and trade secret 
// laws and treaty provisions. No part of the Material may be used, copied, 
// reproduced, modified, published, uploaded, posted, transmitted, distributed, 
// or disclosed in any way without Intels prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
******************************************************************************/

#include <windows.h>
#include <GLES/egl.h>
#include <GLES/gl.h>

#define MAX_LOADSTRING 100

// Global Variables:
HINSTANCE       g_hInst;                // The current instance

EGLDisplay      eglDisplay;
EGLConfig       eglConfig;
EGLContext      eglContext;
EGLSurface      eglWindowSurface;

GLfloat v[] = 
{
        .5f, 0.f, 0.f,
        0.f, 0.f, 0.f,
        0.f, .5f, 0.f,
};
GLfloat fog_Color[4] = {0.3f, 0.3f, 0.3f, 0.5f};


//*****************************************************************************
// Name:            AppInit
// Description:     Application initialization 
//*****************************************************************************
BOOL AppInit( void )
{
    return TRUE;
}



//*****************************************************************************
// Name:            AppDeInit
// Description:     Application release 
//*****************************************************************************
void AppDeInit( void )
{
    // Destroy all EGL resources 
    eglMakeCurrent(eglDisplay, NULL, NULL, NULL);
    eglDestroyContext(eglDisplay, eglContext);
    eglDestroySurface(eglDisplay, eglWindowSurface);
    eglTerminate(eglDisplay);

    return;
}



//*****************************************************************************
// Name:            AppUpdate
// Description:     Application update 
//*****************************************************************************
void AppUpdate( void )
{
    static int mode = 0;

	// Enable blending
    glEnable(GL_BLEND);

	// Set the pixel arithmetic operation used by blending
    switch(mode%4)
    {
    case 0:
        glColor4f(0.0f, 1.0f, 1.0f, 0.1f);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        break;
    case 1:
        glColor4f(0.0f, 0.0f, 1.0f, 0.4f);
        glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
        break;
    case 2:
        glColor4f(0.0f, 0.5f, 0.5f, 0.1f);
        glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
        break;
    case 3:
        glDisable(GL_BLEND);
        break;
    }

	// Change blending mode

    mode ++;
    return;
}



//*****************************************************************************
// Name:            AppRender
// Description:     Application rendering. Three triangles will be rendered abreast and their color 
//                  change according to different blending function setting.   
//*****************************************************************************
void AppRender( void )
{
	// Clear the color buffer with red
    glClearColor(1.f, .0f, 0.0f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	// Set vertex pointer and enable the vertex array for rendering
    glVertexPointer(3, GL_FLOAT, 0, v);
    glEnableClientState( GL_VERTEX_ARRAY);

	// Set projection matrix
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustumf(-2.0f, 2.0f, -1.f, 1.f, 1.f, 7.f);

	// Set model-view matrix
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

	// Draw the triangle for three times
    // For each time a different translation is set
    glPushMatrix();
    glTranslatef(-1.f, 0.f, -1.0f);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glPopMatrix();

    glPushMatrix();
    glTranslatef(0.f, 0.f, -2.0f);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glPopMatrix();

    glPushMatrix();
    glTranslatef(1.f, 0.f, -3.0f);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glPopMatrix();

	// Swap the color buffer to window surface 
    eglSwapBuffers (eglDisplay, eglWindowSurface);

	// Sleep for 500 ms
    Sleep(500);
    return;
}



//*****************************************************************************
// Name:            WndProc
// Description:     Processes messages for the main window 
//*****************************************************************************
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) 
    {
        case WM_KEYDOWN:
        case WM_LBUTTONDOWN:
        case WM_CLOSE:
            DestroyWindow (hWnd);
            break;

        case WM_DESTROY:
            PostQuitMessage (0);
            break;

        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
    }

    return 0;
}

//*****************************************************************************
// Name:            MyRegisterClass
// Description:     Registers the window class 
//*****************************************************************************
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
    WNDCLASS    wc;

    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc      = (WNDPROC) WndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = hInstance;
    wc.hIcon            = NULL;
    wc.hCursor          = 0;
    wc.hbrBackground    = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName     = 0;
    wc.lpszClassName    = szWindowClass;

    return RegisterClass(&wc);
}



//*****************************************************************************
// Name:            InitInstance
// Description:     Save application instance handle and creates main window 
//*****************************************************************************
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    HWND    hWnd = NULL;

    // Store instance handle in our global variable
    g_hInst = hInstance;        

    //If it is already running, then focus on the window
    hWnd = FindWindow(L"OGL-ES Sample", L"OGL-ES Sample");  
    if (hWnd) 
    {
        // set focus to foremost child window
        // The "| 0x01" is used to bring any owned windows to the foreground and
        // activate them.
        SetForegroundWindow((HWND)((ULONG) hWnd | 0x00000001));
        return 0;
    }

    // Register window class
    MyRegisterClass(hInstance, L"OGL-ES Sample");

    // Create main application window
    hWnd = CreateWindow( L"OGL-ES Sample", L"OGL-ES Sample", WS_VISIBLE,
                         0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), 
                         NULL, NULL, hInstance, NULL );
    if (!hWnd)
    {   
        return FALSE;
    }

    // Show main window
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    EGLint      numConfigs;
    EGLint      majorVersion;
    EGLint      minorVersion;
    EGLConfig   config;
    EGLint      attrib_list[] = {EGL_RED_SIZE, 1,
                                 EGL_DEPTH_SIZE, 1,
                                 EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
                                 EGL_NONE
                                };

	// Get EGL display
    eglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    if(eglDisplay == EGL_NO_DISPLAY || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }

	// Initialize EGL
    eglInitialize(eglDisplay, &majorVersion, &minorVersion);
    if(eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }

	// Choose EGL config
    if(!eglChooseConfig(eglDisplay, attrib_list, &config, 1, &numConfigs))
    {
        return FALSE;
    }
    
	// Create EGL rendering context
    eglContext = eglCreateContext(eglDisplay, config, NULL, NULL);
    if(eglContext == EGL_NO_CONTEXT || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }

	// Create EGL window surface
    eglWindowSurface = eglCreateWindowSurface(eglDisplay, config, hWnd, NULL);
    if(eglWindowSurface == EGL_NO_SURFACE || eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }
    
	// Attach the EGL rendering context to EGL surfaces
    eglMakeCurrent(eglDisplay, eglWindowSurface, eglWindowSurface, eglContext);
    if(eglGetError() != EGL_SUCCESS)
    {
        return FALSE;
    }


    return TRUE;
}



//*****************************************************************************
// Name:            WinMain
// Description:     Application entry 
//*****************************************************************************
int WINAPI WinMain (
    HINSTANCE   hInstance, 
    HINSTANCE   hPrevInstance, 
    LPWSTR      lpCmdLine, 
    int         nCmdShow)
{
    MSG msg;

    // Perform application initialization:
    if( !InitInstance (hInstance, nCmdShow) ) 
    {
        return FALSE;
    }

    // Application initialization
    if( !AppInit() )
    {
        return FALSE;
    }

    while( TRUE )
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
        {
            if( WM_QUIT == msg.message )
                break;
            TranslateMessage( &msg );
            DispatchMessage ( &msg );
        }
        else
        {
            // Application update
            AppUpdate();

            // Application render
            AppRender();
        }
    }

    // Application release
    AppDeInit();

    return msg.wParam;
}


// EOF